import argparse
import os
import time
import logging
from bs4 import BeautifulSoup, NavigableString, Tag
from googletrans import Translator
from tqdm import tqdm

def translate_text(text, translator, target_lang):
    try:
        result = translator.translate(text, dest=target_lang)
        return result.text
    except Exception as e:
        logging.warning(f"Translation error: {e}")
        return text

def count_translatable_nodes(soup):
    count = 0
    for node in soup.descendants:
        if isinstance(node, NavigableString):
            if node.strip() and not is_inside_script_or_style(node):
                count += 1
    return count

def is_inside_script_or_style(node):
    parent = node.parent
    while parent:
        if isinstance(parent, Tag) and parent.name in ("script", "style"):
            return True
        parent = parent.parent
    return False

def translate_soup(soup, translator, target_lang, pbar=None):
    def progress_translate_node(node):
        if isinstance(node, NavigableString) and not is_inside_script_or_style(node):
            text = node.strip()
            if text:
                translated = translate_text(text, translator, target_lang)
                time.sleep(0.5)
                if pbar:
                    pbar.update(1)
                return translated
        elif hasattr(node, 'contents'):
            for i, child in enumerate(node.contents):
                if isinstance(child, NavigableString) and not is_inside_script_or_style(child):
                    text = child.strip()
                    if text:
                        translated = translate_text(text, translator, target_lang)
                        time.sleep(0.5)
                        node.contents[i].replace_with(translated)
                        if pbar:
                            pbar.update(1)
                else:
                    progress_translate_node(child)

    progress_translate_node(soup)

def process_file(filepath, translator, lang):
    logging.info(f"Processing file: {filepath}")
    with open(filepath, 'r', encoding='utf-8') as f:
        soup = BeautifulSoup(f, "html.parser")

    count = count_translatable_nodes(soup)
    logging.info(f"Found {count} translatable texts.")

    with tqdm(total=count, desc="Translating", ncols=80) as pbar:
        translate_soup(soup, translator, lang, pbar)

    base, ext = os.path.splitext(filepath)
    output_path = f"{base}_{lang}{ext}"
    with open(output_path, 'w', encoding='utf-8') as f:
        f.write(str(soup))
    logging.info(f"✅ Translated saved to: {output_path}")

def main():
    parser = argparse.ArgumentParser(
        description="🌐 html2lang - Translate HTML files to any language."
    )
    parser.add_argument(
        "-l", "--lang", help="Target language code (e.g. uz, fr, en)", required=False
    )
    parser.add_argument(
        "-f", "--html", help='HTML file name (e.g. index.html) or "all"', required=False
    )
    parser.add_argument(
        "--creator", action="store_true", help="Show creator info"
    )

    args = parser.parse_args()

    if args.creator:
        print("👤 Created by Axmadjon Qaxxorov (@qaxxorov)")
        return

    if not (args.lang and args.html):
        parser.print_help()
        return

    lang = args.lang
    html_arg = args.html

    logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s", level=logging.INFO)
    start = time.time()
    translator = Translator()

    if html_arg == "all":
        current_dir = os.getcwd()
        html_files = [
            os.path.join(root, file)
            for root, _, files in os.walk(current_dir)
            for file in files if file.endswith(".html")
        ]

        logging.info(f"Found {len(html_files)} file(s) in current directory.")

        for i, filepath in enumerate(html_files, 1):
            logging.info(f"\n🔄 ({i}/{len(html_files)}): {os.path.basename(filepath)}")
            process_file(filepath, translator, lang)

    else:
        filepath = os.path.abspath(html_arg)
        if not os.path.exists(filepath):
            logging.error(f"❌ File not found: {filepath}")
            return
        process_file(filepath, translator, lang)

    duration = round(time.time() - start, 2)
    logging.info(f"🏁 Finished in {duration} seconds.")

if __name__ == "__main__":
    main()