import os
import time
import logging
from bs4 import BeautifulSoup, NavigableString
from googletrans import Translator
from tqdm import tqdm

translator = Translator()

def translate_text(text: str, lang: str) -> str:
    try:
        return translator.translate(text, dest=lang).text
    except Exception as e:
        logging.warning(f"Translation failed: {e}")
        return text

def get_translatable_nodes(soup: BeautifulSoup) -> list:
    return [
        node for node in soup.descendants
        if isinstance(node, NavigableString) and node.strip()
    ]

def translate_nodes(nodes: list, lang: str, pbar=None):
    for node in nodes:
        original = node.strip()
        translated = translate_text(original, lang)
        time.sleep(0.5)  # avoid rate limit
        node.replace_with(translated)
        if pbar:
            pbar.update(1)

def translate_html_file(input_path: str, output_path: str, lang: str):
    logging.info(f"📄 Processing: {input_path}")
    with open(input_path, "r", encoding="utf-8") as f:
        soup = BeautifulSoup(f, "html.parser")

    nodes = get_translatable_nodes(soup)
    logging.info(f"🔍 Found {len(nodes)} translatable strings.")

    with tqdm(total=len(nodes), desc="🔠 Translating", ncols=80) as pbar:
        translate_nodes(nodes, lang, pbar)

    os.makedirs(os.path.dirname(output_path), exist_ok=True)
    with open(output_path, "w", encoding="utf-8") as f:
        f.write(str(soup))

    logging.info(f"✅ Saved: {output_path}")

def translate_all_html_files(input_dir: str, output_dir: str, lang: str):
    html_files = [
        os.path.join(root, file)
        for root, _, files in os.walk(input_dir)
        for file in files if file.endswith(".html")
    ]

    logging.info(f"📁 Found {len(html_files)} .html files.")
    for i, input_path in enumerate(html_files, 1):
        rel_path = os.path.relpath(input_path, input_dir)
        output_path = os.path.join(output_dir, rel_path)
        logging.info(f"\n[{i}/{len(html_files)}] Translating: {rel_path}")
        translate_html_file(input_path, output_path, lang)
